/**
 * Sketch: Doorbell
 * Version: 1.0 April 2nd 2016
 * Author: by Theo
 * 
 * 
 * MySensors doorbell sensor. This sensor is a fully working doorbell, that is also connected to the MySensors network. Through MySensors you can:
 * - detect whether the doorbell has been pressed
 * - turn of the doorbell sound (for instance when you or your little child is sleeping.
 * - control the volume of the doorbell
 * 
 * The doorbell is designed around the HK592 which is a real cheap doorbell chip making a double ding dong sound. Other melody chips can be used, but you might
 * need to change the circuit. The HKxxx chips are hard to find. As an alternative you can us an UM348x IC. Which is also a melody chip. However, note that you
 * might not be able to interrupt a melody that's being played bij the UM chip. So in that case the doorbell will play a sound for a longer period as you might
 * like.
 * 
 * Design decision(s):
 * - Must use an melody chip as a source for playing sounds. Reason(s):
 *   1. I want to get to know more about electronic circuits (but just a personal driver)
 *   2. These chips only need a single signal to start playing. Thus it will simplyfy the Arduino code. No checking if we have to end the melody that's being
 *      played or what so ever.
 * - Don't use breakout boards that are able to play MP3's or any other soundboard. Reason(s):
 *   1. See reason 2 for previous decision
 *   2. It'll need extra hardware in terms of amplifier etc. This will raise the total costs of this doorbell.
 * 
 * Requirement(s):
 * 1. The total costs of the doorbell should be as low as possibel. Meaning cheaper than a doorbell sold at any store.
 * 2. Volume control is a must
 * 3. Silent mode is a must
 * 4. Reliability: Doorbell must be working at all times
 * 5. Power consumption must be as low as possible.
 *
 * Revision history:
 * 1.1 April 12th 2016
 *   - added a motion sensor node, for my own convenience. I just needed one at the same spot as the doorbell. Comment #define USE_MOTION_SENSOR if you don't
 *     need one
 * 1.0 April second 2016
 *    - initial version
 */

// Uncomment following line if you want a PIR motion sensor. Use a mini PIR if you connect one to a pro Mini 3.3V. The mini has 3.3V output, the larger one has 5V
#define USE_MOTION_SENSOR

#include <MySensor.h>  
#include <SPI.h>

// ---------------------- constants definitions (pinouts etc)   -----------------------------
#define DOORBELL_SWITCH_PIN  2 // Interrupt pin to which the doorbell switch is connected
#define MOTION_SENSOR_PIN    3 // reserved for PIR sensor, which will be added later on after the doorbell is published on openhardware.io
#define SOUND_PIN            4 // Pin to which the transistor for starting the doorbell sound is connected (Prob a BC547)
#define DOUBLEPRESS_INTERVAL 8000 // The amount of milli seconds before a new press on the doorbell switch will trigger a new sound
#define SILENTMODE_ON        1
#define SILENTMODE_OFF       0
#define DOORBELL_SWITCH_ID   1   // Id of the doorbell switch child
#define DOORBELL_MUTE_ID     2   // Id of the doorbell mute state
#define DOORBELL_CHIME_ID    3   // Id of the doorbell mute state

#ifdef USE_MOTION_SENSOR
#define MOTION_CHILD_ID      4   // Id of the motion sensor
#endif

// ---------------------- State variables for interrupt handling and other doorbell states  -----------------------------
bool doorSwitchStateChanged = false;   // state for interrupt detection of a press on the doorbell switch
bool motionSensorStateChanged = false; // state for interrupt detection of a change to the pir sensor state
bool silentMode = SILENTMODE_OFF;      // state indicating that the silent mode is on or off. Silentmode means no sound is being played.
                                       // only the gateway is being informed of a press on the doorbell switch.

// Variables
unsigned long ignoreDoorbellSwitchDoublePressIntervall = 0; // The time in ms after which a doorbell switch will trigger a sound. Double press protection.
                                                            // it also discourages youngsters from pressing the doorbell and hiding.
MySensor gw;

#ifdef USE_MOTION_SENSOR
byte motionStateChanged = false;
#endif

// Initialize doorbell switch message
MyMessage doorbellSwitchMsg( DOORBELL_SWITCH_ID, V_TRIPPED );
MyMessage chimeMuteStateMsg( DOORBELL_MUTE_ID,   V_LIGHT );
MyMessage doorbellChimeMsg( DOORBELL_CHIME_ID,  V_LIGHT );

#ifdef USE_MOTION_SENSOR
MyMessage motionMsg( MOTION_CHILD_ID, V_TRIPPED );
#endif

// ---------------------- Sketch initialization   -----------------------------
void setup() {
//  Serial.begin( 115200 );

  gw.begin( incomingMessage ); //, 254 ); // 254 is reserved for sensor development.

  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo("Doorbell chime", "1.0");

  pinMode( DOORBELL_SWITCH_PIN, INPUT_PULLUP );
  pinMode( SOUND_PIN, OUTPUT );
  pinMode( MOTION_SENSOR_PIN, INPUT );
  digitalWrite( MOTION_SENSOR_PIN,LOW );
  attachInterrupt( digitalPinToInterrupt( DOORBELL_SWITCH_PIN ), doorbellSwitchChanged, FALLING );

#ifdef USE_MOTION_SENSOR
  attachInterrupt( digitalPinToInterrupt( MOTION_SENSOR_PIN ), handleMotionStateInterrupt, CHANGE );
#endif  
  
  gw.present( DOORBELL_SWITCH_ID, S_MOTION, "Doorbell switch", true );
  gw.wait( 50 ); // apparently the node can't handle this fast presentation of the different nodes.
  gw.present( DOORBELL_MUTE_ID, S_LIGHT, "Doorchime mute", true );
  gw.present( DOORBELL_CHIME_ID,  S_BINARY, "Doorchime" ); // actuators don't need an ACK
  gw.wait( 50 );
#ifdef USE_MOTION_SENSOR
  gw.present(MOTION_CHILD_ID, S_MOTION, "Motion sensor", true );
  gw.wait( 50 );
#endif
  gw.request( DOORBELL_MUTE_ID, V_LIGHT );
  gw.wait( 50 );
  gw.send( doorbellChimeMsg.set( 0 ),true );
  gw.wait( 50 );
  gw.sendBatteryLevel( 100 );
  gw.wait( 50);

#ifdef USE_MOTION_SENSOR
  handleMotionStateInterrupt();
#endif
}

void incomingMessage(const MyMessage &message) {
  if ( message.type == V_LIGHT ) { // Gateway reports a new Light state
    if ( message.sensor == DOORBELL_MUTE_ID ) {
      silentMode = ( message.getBool() == true ? SILENTMODE_OFF : SILENTMODE_ON );
    }
    else if (  message.sensor == DOORBELL_CHIME_ID ) {
      if ( message.getBool() == true ) {
        playDoorbellSound();
        gw.wait( 50 );
        gw.send( doorbellChimeMsg.set( 0 ), true );
      }
    }
  }
}

void playDoorbellSound() {
//    Serial.println( "Ding Dong" );
    digitalWrite( SOUND_PIN, HIGH );
    gw.wait( 100 ); 
    digitalWrite( SOUND_PIN, LOW );
}

// ---------------------- main loop   -----------------------------
void loop() {
  if ( doorSwitchStateChanged == true ) {
    if ( digitalRead( DOORBELL_SWITCH_PIN ) == LOW ) {
      unsigned long pressedTime = millis();
      if ( pressedTime > ignoreDoorbellSwitchDoublePressIntervall ) {
        ignoreDoorbellSwitchDoublePressIntervall = pressedTime + DOUBLEPRESS_INTERVAL;
        gw.send( doorbellSwitchMsg.set( 1 ), true ); // request ACK!!
        if ( silentMode == SILENTMODE_OFF ) {
          playDoorbellSound();
        }
//        else {
//          Serial.println( "Doorbell switch pressed - silent mode." );
//        }
        gw.wait( 50 );
        gw.send( doorbellSwitchMsg.set( 0 ), true ); // request ACK!!
      }
    }
    doorSwitchStateChanged = false;
  }

#ifdef USE_MOTION_SENSOR
  if ( motionSensorStateChanged == true ) {
    motionSensorStateChanged = false;
    handleMotionStateInterrupt();
  }
#endif
  gw.wait( 50 ); // give the input pins some rest. Incomming messages are still being processed.
}

// ---------------------- Interrupt handlers   -----------------------------
/**
 * Interrupt handler for detecting doorbell switch presses
 */
void doorbellSwitchChanged() {
    doorSwitchStateChanged = true;
}

#ifdef USE_MOTION_SENSOR
void handleMotionStateInterrupt() {
  gw.send(motionMsg.set(digitalRead( MOTION_SENSOR_PIN ) == HIGH?"1":"0"));
}
#endif

